/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2008 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include <taag.h>
#include <taag_xml.h>

/*
 *To compile this file using gcc you can type
 *gcc `xml2-config --cflags --libs` -o mapping_generator mapping_generator.c
 */

static int rslFileOpened = TAAG_FALSE;
static FILE *f;
static char fileName[TAAG_MAX_FILENAME_STRING];
//static char ldLibrary[TAAG_MAX_FILENAME_STRING];
static char executable[TAAG_MAX_FILENAME_STRING];
//static char *g_location;
static int isGroups = 0;

static xmlNodePtr generateGroup(int id, int nrProcs, int *counter) {
	xmlNodePtr node = NULL;
	char s_data[TAAG_MAX_HOSTNAME_STRING];
	int size;

	node = xmlNewNode(NULL, BAD_CAST TAAG_XML_GROUP);
	sprintf(s_data, "%d", id);
	xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_ID, BAD_CAST s_data);

	printf("Number of processes in the %d. Group:", id);
	scanf("%d", &size);

	for (int i = 0; i < size && *counter < nrProcs; i++) {
		sprintf(s_data, "%d", *counter);
		xmlNewChild(node, NULL, BAD_CAST TAAG_XML_G_PROCS, BAD_CAST s_data);
		(*counter)++;
	}
	return (node);
}

static xmlNodePtr generateEdge(int id) {
	xmlNodePtr node = NULL;
	char s_data[TAAG_MAX_HOSTNAME_STRING];
	int i_data;

	node = xmlNewNode(NULL, BAD_CAST TAAG_XML_EDGE);
	printf("The one end of the %d. edge:", id);
	scanf("%d", &i_data);
	sprintf(s_data, "%d", i_data);
	xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_FIRST_END, BAD_CAST s_data);

	printf("The other end of the %d. edge:", id);
	scanf("%d", &i_data);
	sprintf(s_data, "%d", i_data);
	xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_OTHER_END, BAD_CAST s_data);

	return (node);
}

static xmlNodePtr generateHost(int lanId, int id, int nrProcs, int *pCounter) {
	xmlNodePtr node = NULL;
	char s_data[TAAG_MAX_HOSTNAME_STRING];
	char hostname[TAAG_MAX_HOSTNAME_STRING];
	char directory[TAAG_MAX_HOSTNAME_STRING];
	int size;
	int cpus;

	node = xmlNewNode(NULL, BAD_CAST TAAG_XML_HOST);
	sprintf(s_data, "%d", id);
	xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_ID, BAD_CAST s_data);

	printf("The host name of the %d. host:", id);
	scanf("%s", hostname);
	xmlNewChild(node, NULL, BAD_CAST TAAG_XML_H_NAME, BAD_CAST hostname);

	printf("Number of CPUs on the host %s:", hostname);
	scanf("%d", &cpus);
	sprintf(s_data, "%d", cpus);
	xmlNewChild(node, NULL, BAD_CAST TAAG_XML_H_CPUS, BAD_CAST s_data);

	printf("Absolute location (dir) of the application on the host %s:", hostname);
	scanf("%s", directory);
	xmlNewChild(node, NULL, BAD_CAST TAAG_XML_DIRECTORY, BAD_CAST directory);

	printf("Number of processes on the host %s:", hostname);
	scanf("%d", &size);

	if (rslFileOpened) {
		fprintf(f,"( &(resourceManagerContact=\"%s\")\n", hostname);
		fprintf(f,"   (count=%d)\n", size);
		fprintf(f,"   (label=\"subjob %d\")\n", id);
		fprintf(f,"   (environment=(GLOBUS_DUROC_SUBJOB_INDEX %d)\n", id);
		fprintf(f,"                (GLOBUS_LAN_ID %d)\n",lanId);
		fprintf(f,"                (LD_LIBRARY_PATH $(GLOBUS_LOCATION)/lib/))\n");
		fprintf(f,"   (arguments=\"%s.xml\")\n", fileName);
		fprintf(f,"   (directory=%s)\n", directory);
		fprintf(f,"   (executable=%s/%s)\n", directory, executable);
		fprintf(f,")\n");
	}

	for (int i = 0; i < size && *pCounter < nrProcs; i++) {
		sprintf(s_data, "%d", *pCounter);
		xmlNewChild(node, NULL, BAD_CAST TAAG_XML_H_PROCS, BAD_CAST s_data);
		(*pCounter)++;
	}
	return (node);
}

static xmlNodePtr generateLAN(int id, int nrProcs, int nrHosts, int *pCounter, int *hCounter) {
	xmlNodePtr node = NULL;
	xmlNodePtr host_node = NULL;
	char s_data[TAAG_MAX_HOSTNAME_STRING];
	int size;

	node = xmlNewNode(NULL, BAD_CAST TAAG_XML_LAN);
	sprintf(s_data, "%d", id);
	xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_ID, BAD_CAST s_data);

	printf("Number of host in the %d. LAN:", id);
	scanf("%d", &size);

	for (int i = 0; i < size && *hCounter < nrHosts; i++) {
		host_node = generateHost(id, *hCounter, nrProcs, pCounter);
		xmlAddChild(node, host_node);
		(*hCounter)++;
	}
	return (node);
}

#if defined(LIBXML_TREE_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)

int main(int argc, char **argv)
{
    xmlDocPtr doc = NULL;       /* document pointer */

    xmlNodePtr root_node = NULL;
    xmlNodePtr graph = NULL;
    xmlNodePtr topology = NULL;
    xmlNodePtr node = NULL;
    xmlDtdPtr dtd = NULL;       /* DTD pointer */
    char s_data[TAAG_MAX_HOSTNAME_STRING];
    int nrProcs, nrGrps, nrEdges;
    int nrLANs, nrHosts;
    int pCounter, hCounter;

    //g_location = getenv("GLOBUS_LOCATION");
    //sprintf(ldLibrary,"%s/lib/",g_location);

    LIBXML_TEST_VERSION;

    doc = xmlNewDoc(BAD_CAST "1.0");
    root_node = xmlNewNode(NULL, BAD_CAST TAAG_XML_ROOT);
    xmlDocSetRootElement(doc, root_node);

    /*
     * Creates a DTD declaration. Isn't mandatory.
     */
    dtd = xmlCreateIntSubset(doc, BAD_CAST TAAG_XML_ROOT, NULL, BAD_CAST "../../etc/taagXML.dtd");

    printf("Application name:");
    scanf("%s", executable);
    xmlNewChild(root_node, NULL, BAD_CAST TAAG_XML_APP_NAME, BAD_CAST executable);

    printf("Application id:");
    scanf("%s", fileName);
    xmlNewChild(root_node, NULL, BAD_CAST TAAG_XML_APP_ID, BAD_CAST fileName);

    printf("Time Stamp (YYYYMMDD):");
    scanf("%s", s_data);
    xmlNewChild(root_node, NULL, BAD_CAST TAAG_XML_TIME_STAMP, BAD_CAST s_data);

    /***** Graph Definition *****/
    graph = xmlNewNode(NULL, BAD_CAST TAAG_XML_GRAPH);
    xmlAddChild(root_node, graph);

    printf("Schema Type (groups/graph/tree/ring):");
    scanf("%s", s_data);
    if (!strcmp(s_data, "groups")) {
    	isGroups = 1;
    }
    xmlNewChild(graph, NULL, BAD_CAST TAAG_XML_GRAPH_TYPE, BAD_CAST s_data);

    printf("Number of Processes:");
    scanf("%d", &nrProcs);
    sprintf(s_data, "%d", nrProcs);
    //xmlNewChild(graph, NULL, BAD_CAST TAAG_XML_NR_PROCS, BAD_CAST s_data);

    printf("Number of Groups:");
    scanf("%d", &nrGrps);
    sprintf(s_data, "%d", nrGrps);
    //xmlNewChild(graph, NULL, BAD_CAST TAAG_XML_NR_GROUPS, BAD_CAST s_data);

    if (!isGroups) {
    	printf("Number of Edges:");
    	scanf("%d", &nrEdges);
    	//sprintf(s_data, "%d", nrEdges);
    	//xmlNewChild(graph, NULL, BAD_CAST TAAG_XML_NR_EDGES, BAD_CAST s_data);
    }
    else {
    	nrEdges = 0;
    }
    pCounter = 0;
    for (int i = 0 ; i < nrGrps; i++) {
    	node = generateGroup(i, nrProcs, &pCounter);
    	xmlAddChild(graph, node);
    }

    for (int i = 0; i < nrEdges; i++) {
    	node = generateEdge(i);
    	xmlAddChild(graph, node);
    }

    /***** Network Topology Definition *****/
    printf("Do you want to describe network topology (yes/no)?");
    scanf("%s", s_data);
    if (!strcmp(s_data, "yes")) {

    	sprintf(s_data,"%s.rsl",fileName);
    	f = fopen (s_data, "w");
    	rslFileOpened = TAAG_TRUE;

		topology = xmlNewNode(NULL, BAD_CAST TAAG_XML_TOPOLOGY);
		xmlAddChild(root_node, topology);

		printf("Number of LANs:");
		scanf("%d", &nrLANs);
		sprintf(s_data, "%d", nrLANs);
		//xmlNewChild(topology, NULL, BAD_CAST TAAG_XML_NR_LANS, BAD_CAST s_data);

		printf("Number of Hosts:");
		scanf("%d", &nrHosts);
		sprintf(s_data, "%d", nrHosts);
		//xmlNewChild(topology, NULL, BAD_CAST TAAG_XML_NR_HOSTS, BAD_CAST s_data);

		if (rslFileOpened) {
			fprintf(f,"+\n");
		}

		pCounter = 0; hCounter = 0;
		for (int i = 0 ; i < nrLANs; i++) {
			node = generateLAN(i, nrProcs, nrHosts, &pCounter, &hCounter);
			xmlAddChild(topology, node);
		 }

		for (int i = 0; i < nrHosts; i++) {
			for (int j = i; j < nrHosts; j++) {
				if (i==j) {
					node = xmlNewChild(topology, NULL, BAD_CAST TAAG_XML_LATENCY, BAD_CAST "0.0");
				} //if
				else {
					node = xmlNewChild(topology, NULL, BAD_CAST TAAG_XML_LATENCY, BAD_CAST "1.0");
				} //else
				sprintf(s_data, "%d", i);
				xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_ROW, BAD_CAST s_data);
				sprintf(s_data, "%d", j);
				xmlNewProp(node, BAD_CAST TAAG_XML_ATTR_COLUMN, BAD_CAST s_data);
			} //for
		} //for

		//***** Close RSL file
		rslFileOpened = TAAG_FALSE;
		fclose(f);
    } //if

    /*
     * Dumping document to stdio or file
     */
    sprintf(s_data,"%s.xml",fileName);
    xmlSaveFormatFileEnc(s_data, doc, "UTF-8", 1);

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    /*
     * this is to debug memory for regression tests
     */
    xmlMemoryDump();
    return(0);
}
#else
int main(void) {
    fprintf(stderr, "tree support was not compiled in GNU libxml2 library\n");
    exit(1);
}
#endif
