/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2010 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#ifndef PROCSDISTRIBUTION_H_
#define PROCSDISTRIBUTION_H_

#include <stdlib.h>
#include <vector>
#include "taag_basic.h"
#include "GroupData.h"
#include "LatencyMatrix.h"

using namespace std;

extern double overallAvgOfLatencies;

class ProcsDistribution {
	private :
		vector<GroupData *> grpList;
		bool mapped;
		double maxLatencyAmongGrps, maxTmpAmongGrps;
		double avgLatencyAmongGrps, avgTmpAmongGrps;
		int greatestGroupLatencyLevel, greatestGroupTmpLevel;

		void setMapped();
		void calculateMaxLatencyAmongGrps(enum _schema_types type, LatencyMatrix *matrix);
		void calculateGreatestGroupLatencyLevel();

		bool *edges;

	public :
		ProcsDistribution();
		virtual ~ProcsDistribution();
		int addGroup(GroupData *newGrp);
		int getNrOfGroups();
		GroupData *getGroupByIndex(unsigned int index);
		GroupData *getLastGroup();
		void removeLastGroup();

		void initEdgeMatrix();
		void setEdge(int grp1, int grp2);
		bool getEdge(int grp1, int grp2);
		bool areEdgesDefined();


		double mappingQuality();
		double tmpQuality();
		bool isMapped();
		void printOutDistribution(FILE *f);
		void printOutMapping(FILE *f);
		bool compareAlternativeMappings(enum _schema_types type, LatencyMatrix *matrix);
		void resetLastCheckedLCEntries();
};

#endif /* PROCSDISTRIBUTION_H_ */
