/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2010 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

// This is a scalable virtual topology

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <taag_basic.h>
#include <taag_errno.h>

#include "HostData.h"
#include "ListOfHosts.h"
#include "LatencyMatrix.h"

#define NR_OF_HOSTS 100 //50 100 200 300, the minimum is 50

/**********************************************************
 * If we assume that 0.11s is the lowest latency value then
 * 1. level 0.11-0.44	(x4)
 * 2. level 0.44-1.54	(x3.5)
 * 3. level 1.54-4.62   (x3)
 * 4. level 4.62-11.55  (x2.5)
 * 5. level 11.55-23.11 (x2)
 **********************************************************/

int _nws_setNameServer(char *_nwsNameServer, unsigned int _nwsNameServerPort) {
	return TAAG_SUCCESS;
}

int _nws_getHosts(ListOfHosts *hostList, int factor) {
	HostData *host;
	char hostname[256];
	int clusterSize=NR_OF_HOSTS/5; //20 with 1 CPU
	int counter = 0;
	for (int i= counter; i<clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 1.0, 1, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/4; //5 with 4 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 4.0, 4, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/4*3; //15 with 1 CPU
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 1.0, 1, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/10;	//2 with 16 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 16.0, 16, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/10*3;	//6 with 4 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 4.0, 4, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/10*6;	//12 with 2 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 2.0, 2, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/2;	//10 with 1 CPU
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 1.0, 1, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/2;	//10 with 2 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 2.0, 2, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/5;	//4 with 8 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 8.0, 8, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/5*2;	//8 with 4 CPUs
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 4.0, 4, factor);
		hostList->addHost(host);
	}

	counter+=clusterSize;
	clusterSize = (NR_OF_HOSTS/5)/5*2;	//8 with 1 CPU
	for (int i= counter; i<counter+clusterSize; i++) {
		sprintf(hostname,"H%d",i);
		host = new HostData(i, hostname, 1.0, 1, factor);
		hostList->addHost(host);
	}

	return TAAG_SUCCESS;
}

int _nws_getLatencies(int nrOfHosts, LatencyMatrix *matrix) {
	int clusterSize1=NR_OF_HOSTS/5; //20 with 1 CPU
	int counter1 = 0;
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=NR_OF_HOSTS/5; //20 with 1 CPU
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.11, 0.11, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/4; //5 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.7, 1.7, i, j);  //third level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/4*3; //15 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.7, 1.7, i, j);  //third level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10; //2 with 16 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/4; //5 with 4 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/4; //5 with 4 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.09, 0.09, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/4*3; //15 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.6, 0.6, i, j);  //second level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10; //2 with 16 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i

	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/4*3; //15 with 1 CPU
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/4*3; //15 with 1 CPU
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.09, 0.09, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10; //2 with 16 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(7.00, 7.00, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/10; //2 with 16 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/10; //2 with 16 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.10, 0.10, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.3, 1.3, i, j); //BK_TEST 0.15 //first level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(2.5, 2.5, i, j);  //third level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/10*3; //6 with 4 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.10, 0.10, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(2.5, 2.5, i, j);  //third level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/10*6; //12 with 2 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.14, 0.14, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(12.00, 12.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(13.00, 13.00, i, j);  //fifth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/2; //10 with 1 CPU
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/2; //10 with 1 CPU
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.13, 0.13, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.5, 0.5, i, j);  //second level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/2; //10 with 2 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/2; //10 with 2 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.13, 0.13, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(5.5, 5.5, i, j);  //fourth level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/5; //4 with 8 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/5; //4 with 8 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.1, 0.1, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.2, 1.2, i, j);  //BK_TEST 0.3 //first level
		}

		counter2 +=clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.6, 1.6, i, j);  //third level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/5*2; //8 with 4 CPUs
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.10, 0.10, i, j);  //first level
		}

		counter2 =counter1+clusterSize2;
		clusterSize2 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		for (int j = counter2; j<counter2+clusterSize2; j++) {
			if (i!=j) matrix->initValue(1.6, 1.6, i, j);  //third level
		}
	} //for i
	//--------------------------------------------------------------
	counter1 += clusterSize1;
	clusterSize1 = (NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
	for (int i = counter1; i<counter1+clusterSize1; i++) {
		int clusterSize2=(NR_OF_HOSTS/5)/5*2; //8 with 1 CPU
		int counter2 = counter1+1;
		for (int j = counter2; j<counter1+clusterSize2; j++) {
			if (i!=j) matrix->initValue(0.2, 0.2, i, j);  //first level
		}
	} //for i
	return TAAG_SUCCESS;
}
